@@ -1,4 +1,4 @@
-from test_support import verbose, TESTFN
+from test.support import verbose, TESTFN
 import random
 import os
 
@@ -36,7 +36,7 @@ dict2 = {}
 dict1keys = []
 dict2keys = []
 
-# Global flag telling maybe_mutate() wether to *consider* mutating.
+# Global flag telling maybe_mutate() whether to *consider* mutating.
 mutate = 0
 
 # If global mutate is true, consider mutating a dict.  May or may not
@@ -69,14 +69,12 @@ def maybe_mutate():
 
     elif keys:
         # Delete a key at random.
+        mutate = 0   # disable mutation until key deleted
         i = random.randrange(len(keys))
         key = keys[i]
         del target[key]
-        # CAUTION:  don't use keys.remove(key) here.  Or do <wink>.  The
-        # point is that .remove() would trigger more comparisons, and so
-        # also more calls to this routine.  We're mutating often enough
-        # without that.
         del keys[i]
+        mutate = 1
 
 # A horrid class that triggers random mutations of dict1 and dict2 when
 # instances are compared.
@@ -90,14 +88,21 @@ class Horrid:
         # have any systematic relationship between comparison outcomes
         # (based on self.i and other.i) and relative position within the
         # hash vector (based on hashcode).
-        self.hashcode = random.randrange(1000000000)
+        # XXX This is no longer effective.
+        ##self.hashcode = random.randrange(1000000000)
 
     def __hash__(self):
+        return 42
         return self.hashcode
 
-    def __cmp__(self, other):
+    def __eq__(self, other):
         maybe_mutate()   # The point of the test.
-        return cmp(self.i, other.i)
+        return self.i == other.i
+
+    def __ne__(self, other):
+        raise RuntimeError("I didn't expect some kind of Spanish inquisition!")
+
+    __lt__ = __le__ = __gt__ = __ge__ = __ne__
 
     def __repr__(self):
         return "Horrid(%d)" % self.i
@@ -108,10 +113,10 @@ class Horrid:
 
 def fill_dict(d, candidates, numentries):
     d.clear()
-    for i in xrange(numentries):
+    for i in range(numentries):
         d[Horrid(random.choice(candidates))] = \
             Horrid(random.choice(candidates))
-    return d.keys()
+    return list(d.keys())
 
 # Test one pair of randomly generated dicts, each with n entries.
 # Note that dict comparison is trivial if they don't have the same number
@@ -130,13 +135,13 @@ def test_one(n):
     # same size.
     mutate = 1
     if verbose:
-        print "trying w/ lengths", len(dict1), len(dict2),
+        print("trying w/ lengths", len(dict1), len(dict2), end=' ')
     while dict1 and len(dict1) == len(dict2):
         if verbose:
-            print ".",
-        c = cmp(dict1, dict2)
+            print(".", end=' ')
+        c = dict1 == dict2
     if verbose:
-        print
+        print()
 
 # Run test_one n times.  At the start (before the bugs were fixed), 20
 # consecutive runs of this test each blew up on or before the sixth time
@@ -147,7 +152,7 @@ def test_one(n):
 # leak).
 
 def test(n):
-    for i in xrange(n):
+    for i in range(n):
         test_one(random.randrange(1, 100))
 
 # See last comment block for clues about good values for n.
@@ -181,7 +186,7 @@ class Parent:
 # the expected-output file doesn't need to change.
 
 f = open(TESTFN, "w")
-print >> f, Parent().__dict__
+print(Parent().__dict__, file=f)
 f.close()
 os.unlink(TESTFN)
 
@@ -202,16 +207,16 @@ class Machiavelli:
 
         # Michael sez:  "doesn't crash without this.  don't know why."
         # Tim sez:  "luck of the draw; crashes with or without for me."
-        print >> f
+        print(file=f)
 
-        return `"machiavelli"`
+        return repr("machiavelli")
 
     def __hash__(self):
         return 0
 
 dict[Machiavelli()] = Machiavelli()
 
-print >> f, str(dict)
+print(str(dict), file=f)
 f.close()
 os.unlink(TESTFN)
 del f, dict
@@ -275,7 +280,7 @@ dict[Machiavelli3(2)] = Machiavelli3(0)
 f = open(TESTFN, "w")
 try:
     try:
-        print >> f, dict[Machiavelli3(2)]
+        print(dict[Machiavelli3(2)], file=f)
     except KeyError:
         pass
 finally:
@@ -283,3 +288,4 @@ finally:
     os.unlink(TESTFN)
 
 del dict
+del dict1, dict2, dict1keys, dict2keys
