@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
 import unittest
+import sys
 from test import support
 
 class PEP3131Test(unittest.TestCase):
@@ -9,15 +9,26 @@ class PEP3131Test(unittest.TestCase):
             ä = 1
             µ = 2 # this is a compatibility character
             蟒 = 3
-        self.assertEquals(getattr(T, "\xe4"), 1)
-        self.assertEquals(getattr(T, "\u03bc"), 2)
-        self.assertEquals(getattr(T, '\u87d2'), 3)
+            x󠄀 = 4
+        self.assertEqual(getattr(T, "\xe4"), 1)
+        self.assertEqual(getattr(T, "\u03bc"), 2)
+        self.assertEqual(getattr(T, '\u87d2'), 3)
+        self.assertEqual(getattr(T, 'x\U000E0100'), 4)
+
+    def test_non_bmp_normalized(self):
+        𝔘𝔫𝔦𝔠𝔬𝔡𝔢 = 1
+        # On wide builds, this is normalized, but on narrow ones it is not. See
+        # #12746.
+        try:
+            self.assertIn("𝔘𝔫𝔦𝔠𝔬𝔡𝔢", dir())
+        except AssertionError:
+            raise unittest.case._ExpectedFailure(sys.exc_info())
 
     def test_invalid(self):
         try:
             from test import badsyntax_3131
         except SyntaxError as s:
-            self.assertEquals(str(s),
+            self.assertEqual(str(s),
               "invalid character in identifier (badsyntax_3131.py, line 2)")
         else:
             self.fail("expected exception didn't occur")
