@@ -1,7 +1,6 @@
 #include "Python.h"
 
 #include "code.h"
-#include "compile.h"
 #include "Python-ast.h"
 #include "symtable.h"
 
@@ -71,11 +70,7 @@ PyInit__symtable(void)
     PyModule_AddIntConstant(m, "DEF_GLOBAL", DEF_GLOBAL);
     PyModule_AddIntConstant(m, "DEF_LOCAL", DEF_LOCAL);
     PyModule_AddIntConstant(m, "DEF_PARAM", DEF_PARAM);
-    PyModule_AddIntConstant(m, "DEF_STAR", DEF_STAR);
-    PyModule_AddIntConstant(m, "DEF_DOUBLESTAR", DEF_DOUBLESTAR);
-    PyModule_AddIntConstant(m, "DEF_INTUPLE", DEF_INTUPLE);
     PyModule_AddIntConstant(m, "DEF_FREE", DEF_FREE);
-    PyModule_AddIntConstant(m, "DEF_FREE_GLOBAL", DEF_FREE_GLOBAL);
     PyModule_AddIntConstant(m, "DEF_FREE_CLASS", DEF_FREE_CLASS);
     PyModule_AddIntConstant(m, "DEF_IMPORT", DEF_IMPORT);
     PyModule_AddIntConstant(m, "DEF_BOUND", DEF_BOUND);
