@@ -31,17 +31,19 @@ frame_getlocals(PyFrameObject *f, void *closure)
     return f->f_locals;
 }
 
+int
+PyFrame_GetLineNumber(PyFrameObject *f)
+{
+    if (f->f_trace)
+        return f->f_lineno;
+    else
+        return PyCode_Addr2Line(f->f_code, f->f_lasti);
+}
+
 static PyObject *
 frame_getlineno(PyFrameObject *f, void *closure)
 {
-    int lineno;
-
-    if (f->f_trace)
-        lineno = f->f_lineno;
-    else
-        lineno = PyCode_Addr2Line(f->f_code, f->f_lasti);
-
-    return PyLong_FromLong(lineno);
+    return PyLong_FromLong(PyFrame_GetLineNumber(f));
 }
 
 /* Setter for f_lineno - you can set f_lineno from within a trace function in
@@ -345,16 +347,14 @@ frame_gettrace(PyFrameObject *f, void *closure)
 static int
 frame_settrace(PyFrameObject *f, PyObject* v, void *closure)
 {
+    PyObject* old_value;
+
     /* We rely on f_lineno being accurate when f_trace is set. */
+    f->f_lineno = PyFrame_GetLineNumber(f);
 
-    PyObject* old_value = f->f_trace;
-
+    old_value = f->f_trace;
     Py_XINCREF(v);
     f->f_trace = v;
-
-    if (v != NULL)
-        f->f_lineno = PyCode_Addr2Line(f->f_code, f->f_lasti);
-
     Py_XDECREF(old_value);
 
     return 0;
@@ -391,7 +391,7 @@ static PyGetSetDef frame_getsetlist[] = {
        the local variables in f_localsplus are NULL.
 
    2. We also maintain a separate free list of stack frames (just like
-   integers are allocated in a special way -- see intobject.c).  When
+   floats are allocated in a special way -- see floatobject.c).  When
    a stack frame is on the free list, only the following members have
    a meaning:
     ob_type             == &Frametype
