@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 
 """Freeze a Python script into a binary.
 
@@ -42,7 +42,14 @@ Options:
 
 -h:           Print this help message.
 
--x module     Exclude the specified module.
+-x module     Exclude the specified module. It will still be imported
+              by the frozen binary if it exists on the host system.
+
+-X module     Like -x, except the module can never be imported by
+              the frozen binary.
+
+-E:           Freeze will fail if any modules can't be found (that
+              were not excluded using -x or -X).
 
 -i filename:  Include a file with additional command line options.  Used
               to prevent command lines growing beyond the capabilities of
@@ -50,15 +57,15 @@ Options:
               are read and the -i option replaced with the parsed
               params (note - quoting args in this file is NOT supported)
 
--s subsystem: Specify the subsystem (For Windows only.); 
+-s subsystem: Specify the subsystem (For Windows only.);
               'console' (default), 'windows', 'service' or 'com_dll'
-              
+
 -w:           Toggle Windows (NT or 95) behavior.
               (For debugging only -- on a win32 platform, win32 behavior
               is automatic.)
 
 -r prefix=f:  Replace path prefix.
-              Replace prefix with f in the source path references 
+              Replace prefix with f in the source path references
               contained in the resulting binary.
 
 Arguments:
@@ -82,16 +89,15 @@ if it does, the resulting binary is not self-contained.
 
 # Import standard modules
 
+import modulefinder
 import getopt
 import os
-import string
 import sys
 
 
 # Import the freeze-private modules
 
 import checkextensions
-import modulefinder
 import makeconfig
 import makefreeze
 import makemakefile
@@ -114,13 +120,15 @@ def main():
     odir = ''
     win = sys.platform[:3] == 'win'
     replace_paths = []                  # settable with -r option
+    error_if_any_missing = 0
 
     # default the exclude list for each platform
     if win: exclude = exclude + [
-        'dos', 'dospath', 'mac', 'macpath', 'macfs', 'MACFS', 'posix', 'os2', 'ce']
+        'dos', 'dospath', 'mac', 'macpath', 'macfs', 'MACFS', 'posix',
+        'os2', 'ce',
+        ]
 
-    # modules that are imported by the Python runtime
-    implicits = ["site", "exceptions"]
+    fail_import = exclude[:]
 
     # output files
     frozen_c = 'frozen.c'
@@ -129,14 +137,17 @@ def main():
     makefile = 'Makefile'
     subsystem = 'console'
 
-    # parse command line by first replacing any "-i" options with the file contents.
+    # parse command line by first replacing any "-i" options with the
+    # file contents.
     pos = 1
-    while pos < len(sys.argv)-1: # last option can not be "-i", so this ensures "pos+1" is in range!
+    while pos < len(sys.argv)-1:
+        # last option can not be "-i", so this ensures "pos+1" is in range!
         if sys.argv[pos] == '-i':
             try:
-                options = string.split(open(sys.argv[pos+1]).read())
-            except IOError, why:
-                usage("File name '%s' specified with the -i option can not be read - %s" % (sys.argv[pos+1], why) )
+                options = open(sys.argv[pos+1]).read().split()
+            except IOError as why:
+                usage("File name '%s' specified with the -i option "
+                      "can not be read - %s" % (sys.argv[pos+1], why) )
             # Replace the '-i' and the filename with the read params.
             sys.argv[pos:pos+2] = options
             pos = pos + len(options) - 1 # Skip the name and the included args.
@@ -144,14 +155,14 @@ def main():
 
     # Now parse the command line with the extras inserted.
     try:
-        opts, args = getopt.getopt(sys.argv[1:], 'r:a:de:hmo:p:P:qs:wx:l:')
-    except getopt.error, msg:
+        opts, args = getopt.getopt(sys.argv[1:], 'r:a:dEe:hmo:p:P:qs:wX:x:l:')
+    except getopt.error as msg:
         usage('getopt error: ' + str(msg))
 
     # proces option arguments
     for o, a in opts:
         if o == '-h':
-            print __doc__
+            print(__doc__)
             return
         if o == '-d':
             debug = debug + 1
@@ -175,14 +186,25 @@ def main():
             subsystem = a
         if o == '-x':
             exclude.append(a)
+        if o == '-X':
+            exclude.append(a)
+            fail_import.append(a)
+        if o == '-E':
+            error_if_any_missing = 1
         if o == '-l':
             addn_link.append(a)
         if o == '-a':
-            apply(modulefinder.AddPackagePath, tuple(string.split(a,"=", 2)))
+            modulefinder.AddPackagePath(*a.split("=", 2))
         if o == '-r':
-            f,r = string.split(a,"=", 2)
+            f,r = a.split("=", 2)
             replace_paths.append( (f,r) )
 
+    # modules that are imported by the Python runtime
+    implicits = []
+    for module in ('site', 'warnings', 'encodings.utf_8', 'encodings.latin_1'):
+        if module not in exclude:
+            implicits.append(module)
+
     # default prefix and exec_prefix
     if not exec_prefix:
         if prefix:
@@ -200,13 +222,13 @@ def main():
     if win:
         extensions_c = 'frozen_extensions.c'
     if ishome:
-        print "(Using Python source directory)"
+        print("(Using Python source directory)")
         binlib = exec_prefix
         incldir = os.path.join(prefix, 'Include')
         config_h_dir = exec_prefix
         config_c_in = os.path.join(prefix, 'Modules', 'config.c.in')
         frozenmain_c = os.path.join(prefix, 'Python', 'frozenmain.c')
-        makefile_in = os.path.join(exec_prefix, 'Modules', 'Makefile')
+        makefile_in = os.path.join(exec_prefix, 'Makefile')
         if win:
             frozendllmain_c = os.path.join(exec_prefix, 'Pc\\frozen_dllmain.c')
     else:
@@ -225,7 +247,9 @@ def main():
 
     # sanity check of directories and files
     check_dirs = [prefix, exec_prefix, binlib, incldir]
-    if not win: check_dirs = check_dirs + extensions # These are not directories on Windows.
+    if not win:
+        # These are not directories on Windows.
+        check_dirs = check_dirs + extensions
     for dir in check_dirs:
         if not os.path.exists(dir):
             usage('needed directory %s not found' % dir)
@@ -286,8 +310,8 @@ def main():
     if odir and not os.path.isdir(odir):
         try:
             os.mkdir(odir)
-            print "Created output directory", odir
-        except os.error, msg:
+            print("Created output directory", odir)
+        except os.error as msg:
             usage('%s: mkdir failed (%s)' % (odir, str(msg)))
     base = ''
     if odir:
@@ -309,9 +333,9 @@ def main():
         try:
             custom_entry_point, python_entry_is_main = \
                 winmakemakefile.get_custom_entry_point(subsystem)
-        except ValueError, why:
+        except ValueError as why:
             usage(why)
-            
+
 
     # Actual work starts here...
 
@@ -319,7 +343,7 @@ def main():
     dir = os.path.dirname(scriptfile)
     path[0] = dir
     mf = modulefinder.ModuleFinder(path, debug, exclude, replace_paths)
-    
+
     if win and subsystem=='service':
         # If a Windows service, then add the "built-in" module.
         mod = mf.add_module("servicemanager")
@@ -347,17 +371,22 @@ def main():
 
     if debug > 0:
         mf.report()
-        print
+        print()
     dict = mf.modules
 
+    if error_if_any_missing:
+        missing = mf.any_missing()
+        if missing:
+            sys.exit("There are some missing modules: %r" % missing)
+
     # generate output for frozen modules
-    files = makefreeze.makefreeze(base, dict, debug, custom_entry_point)
+    files = makefreeze.makefreeze(base, dict, debug, custom_entry_point,
+                                  fail_import)
 
     # look for unfrozen modules (builtin and of unknown origin)
     builtins = []
     unknown = []
-    mods = dict.keys()
-    mods.sort()
+    mods = sorted(dict.keys())
     for mod in mods:
         if dict[mod].__code__:
             continue
@@ -381,7 +410,7 @@ def main():
         else:
             # Do the windows thang...
             import checkextensions_win32
-            # Get a list of CExtension instances, each describing a module 
+            # Get a list of CExtension instances, each describing a module
             # (including its source files)
             frozen_extensions = checkextensions_win32.checkextensions(
                 unknown, extensions, prefix)
@@ -391,7 +420,7 @@ def main():
     # report unknown modules
     if unknown:
         sys.stderr.write('Warning: unknown modules remain: %s\n' %
-                         string.join(unknown))
+                         ' '.join(unknown))
 
     # windows gets different treatment
     if win:
@@ -424,17 +453,19 @@ def main():
         outfp.close()
     infp.close()
 
-    cflags = defines + includes + ['$(OPT)']
+    cflags = ['$(OPT)']
+    cppflags = defines + includes
     libs = [os.path.join(binlib, 'libpython$(VERSION).a')]
 
     somevars = {}
     if os.path.exists(makefile_in):
         makevars = parsesetup.getmakevars(makefile_in)
-    for key in makevars.keys():
-        somevars[key] = makevars[key]
+        for key in makevars:
+            somevars[key] = makevars[key]
 
-    somevars['CFLAGS'] = string.join(cflags) # override
-    files = ['$(OPT)', '$(LDFLAGS)', base_config_c, base_frozen_c] + \
+    somevars['CFLAGS'] = ' '.join(cflags) # override
+    somevars['CPPFLAGS'] = ' '.join(cppflags) # override
+    files = [base_config_c, base_frozen_c] + \
             files + supp_sources +  addfiles + libs + \
             ['$(MODLIBS)', '$(LIBS)', '$(SYSLIBS)']
 
@@ -447,18 +478,18 @@ def main():
     # Done!
 
     if odir:
-        print 'Now run "make" in', odir,
-        print 'to build the target:', base_target
+        print('Now run "make" in', odir, end=' ')
+        print('to build the target:', base_target)
     else:
-        print 'Now run "make" to build the target:', base_target
+        print('Now run "make" to build the target:', base_target)
 
 
 # Print usage message and exit
 
 def usage(msg):
     sys.stdout = sys.stderr
-    print "Error:", msg
-    print "Use ``%s -h'' for help" % sys.argv[0]
+    print("Error:", msg)
+    print("Use ``%s -h'' for help" % sys.argv[0])
     sys.exit(2)
 
 
